<?php
/* header.php
Copyright 2006  Didier LOISEAU
Copyright 2006  Jean-Michel PHILIPPE
All rights reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

class ThemeManager
{
	var $_themes_path;
	var $_themes_url;
	var $_user_theme;
	var $_imported_files = array('main.css');
	var $_IE_versions = array(
		'5.0'	=> 'lt IE 5.5000',
		'5.5'	=> 'IE 5.5000',
		'6.0'	=> 'IE 6',
		'7.0'	=> 'IE 7'
	);
	var $_requested_actions = array();
	var $_bodyclass = '';

    function ThemeManager($themes_path, $themes_dir_url, $user_theme = null) {
    	$this->_user_theme = $user_theme;
    	if ($themes_path[strlen($themes_path) - 1] != '/') $themes_path .= '/';
    	$this->_themes_path = $themes_path;
    	if ($themes_dir_url[strlen($themes_dir_url) - 1] != '/') $themes_dir_url .= '/';
    	$this->_themes_url = $themes_dir_url;
    }
    
    function setUserTheme($user_theme)
    {
        // Check theme validity (security check)
        if (!preg_match('/^[a-zA-Z0-9_-]+$/', $user_theme))
        {
            return false;
        }
        if (!file_exists($this->_themes_path . $user_theme . '/main.css'))
        {
            return false;
        }
    	$this->_user_theme = $user_theme;
    }
    
    function getUserTheme()
    {
    	return $this->_user_theme;
    }
    
    function addCSS($name, $module_type = null)
    {
    	if ($module_type !== null)
    	{
    		$this->_imported_files[] = $module_type . '/' . $name . CSS_EX;
    	}
    	else
    	{
    		$this->_imported_files[] = $name . CSS_EX;
    	}
    }
    
	function addActionCSS($actionname)
	{
		$actionname = strtolower($actionname);
		if (!array_key_exists($actionname, $this->_requested_actions))
		{	// check action styles aren't already registered
			$this->_requested_actions[$actionname] = True;
			$this->addCSS($actionname, 'actions');
		}
	}
	
	/**
	 * Resets the list of actions CSS files to be included and the body tag class.
	 * @param string $classname The first body tag class.
	 * @example
	 * $this->resetCSS($classname)
	 */
	function resetCSS($classname)
	{
		$this->_bodyclass = $classname;
		$this->_requested_actions = array();
	}
	
	/**
	 * Adds a class name to the html 'body' tag. No check performed against classes
	 * that would have already been added. No leading space required in class names.
	 * @param string $classname The additional body tag class.
	 * @example
	 * $this->addBodyClass('edit')
	 */
	function addBodyClass($classname)
	{
		$this->_bodyclass .= ' '.$classname;
	}
	
	/**
	 * Returns the file URL of the requested CSS file from the
	 * current theme or from the default theme depending on which
	 * one exists.
	 * @param string $file The CSS file name eventually with its subdirectory
	 * @return The file URL or null.
	 * @example
	 * $this->getFileURL('main.css')
	 * $this->getFileURL('handlers/edit.css')
	 */
    function getFileURL($file)
    {
    	if ($this->_user_theme !== null)
    	{
    		$css_file = $this->_user_theme . '/' . $file;
    		if (file_exists($this->_themes_path . $css_file))
    		{
    			return $this->_themes_url . $css_file;
    		}
    	}
    	$css_file = DEFAULT_THEME . '/' . $file;
    	if (file_exists($this->_themes_path . $css_file))
    	{
    		return $this->_themes_url . $css_file;
    	}
    	return null;
    }
    
	/**
	 * Generates the CSS @import rules to be used in the Html header.
	 * @param string $handler_name The name of the handler being used to display the page
	 * @return The text of @import rules.
	 * @example
	 * $this->generateImports('show')
	 */
    function generateImports($handler_name)
    {
    	$result = '';
    	$file_list = $this->_imported_files;
    	$file_list[] = 'handlers/' . $handler_name . CSS_EX;	// add handler CSS file
    	foreach ($file_list as $file)
    	{
    		$file_url = $this->getFileURL($file);
    		if ($file_url !== null)
	    		$result .= "\t\t@import url(\"" . $file_url . "\");\n";
    	}
    	return $result;
    }
    
    function _getIETricksFiles($theme)
    {
    	$result = array();
    	$theme_path = $this->_themes_path . $theme . '/';
    	$sufix = 'Fixes' . CSS_EX;
    	
    	foreach (array_keys($this->_IE_versions) as $version)
    	{
    		$file = 'IE' . str_replace('.', '', $version) . $sufix;
    		if (file_exists($theme_path . $file))
    		{
    			$result[$version] = $file;
    		}
    	}
    	
    	return $result;
    }
    
    function generateIETricks()
    {
    	$theme = $this->_user_theme !== null ? $this->_user_theme : DEFAULT_THEME;
    	
    	$result = '';
    	$prefix = $this->_themes_url . $theme . '/';
    	foreach ($this->_getIETricksFiles($theme) as $version => $file)
    	{
    		$result .= '	<!--[if '. $this->_IE_versions[$version] ."]>\n"
    			. "		<style type=\"text/css\" media=\"all\">\n"
    			. '			@import url("' . $prefix . $file . "\");\n"
    			. "		</style>\n	<![endif]-->\n";
    	}
    	
    	return $result;
    }
    
    function getThemesList()
    {
    	$th_path = $this->_themes_path;
    	$dir = @opendir($th_path);
    	if (!$dir) return array();
    	
    	$result = array();
    	while (false !== ($theme = readdir($dir)))
    	{
    		if (is_dir($th_path . $theme) && file_exists($th_path . $theme . '/main.css'))
    		{
    			$result[] = $theme;
    		}
    	}
    	
    	return $result;
    }
	
	function getBodyClass()
	{
		return htmlentities($this->_bodyclass);
	}
}
?>